@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import re
 from collections import OrderedDict
-from typing import Any, Optional, Union, MutableSequence, overload, Iterable, TypeVar
+from typing import Any, Optional, Union, Iterable, Tuple, overload, TypeVar
 from dataclasses import dataclass
 import dataclasses
 from abc import ABCMeta
@@ -11,6 +11,15 @@ from . import printing
 from .converters import toKdlValue
 
 
+class _MISSING:
+    pass
+
+
+VT = TypeVar("VT")
+
+NodeKey = Union[str, Tuple[Optional[str], Optional[str]]]
+
+
 @dataclass
 class Document:
     nodes: list[Node] = dataclasses.field(default_factory=list)
@@ -27,6 +36,56 @@ class Document:
             s = "\n"
         return s
 
+    @overload
+    def get(self, key: NodeKey) -> Optional[Node]:
+        ...
+
+    @overload
+    def get(self, key: NodeKey, default: VT) -> Union[Node, VT]:
+        ...
+
+    def get(
+        self,
+        key: NodeKey,
+        default: Any = None,
+    ) -> Any:
+        tag, name, passedTag = tupleFromNodeKey(key)
+        for node in self.nodes:
+            if name is not None and node.name != name:
+                continue
+            if passedTag and node.tag != tag:
+                continue
+            return node
+        return default
+
+    def __getitem__(self, key: NodeKey) -> Node:
+        tag, name, passedTag = tupleFromNodeKey(key)
+        for node in self.nodes:
+            if name is not None and node.name != name:
+                continue
+            if passedTag and node.tag != tag:
+                continue
+            return node
+        if not passedTag:
+            s = f"name '{name}'"
+        elif name is None:
+            s = f"tag '{tag}'"
+        else:
+            s = f"name '{name}' and tag '{tag}'"
+        raise KeyError(f"No node with {s}.")
+
+    def getAll(
+        self,
+        key: NodeKey,
+    ) -> Iterable[Node]:
+        tag, name, passedTag = tupleFromNodeKey(key)
+        for node in self.nodes:
+            if name is not None and node.name != name:
+                continue
+            if passedTag and node.tag != tag:
+                continue
+            yield node
+
     def __str__(self) -> str:
         return self.print()
 
@@ -61,7 +120,12 @@ class Node:
                 continue
             s += f" {printValue(arg, config)}"
 
-        for key, value in sorted(self.props.items(), key=lambda x: x[0]):
+        props: Iterable[Tuple[str, Any]]
+        if config.sortProperties:
+            props = sorted(self.props.items(), key=lambda x: x[0])
+        else:
+            props = self.props.items()
+        for key, value in props:
             value = toKdlValue(value)
             if not config.printNullProps and (value is None or isinstance(value, Null)):
                 continue
@@ -82,6 +146,56 @@ class Node:
             s += "\n"
         return s
 
+    @overload
+    def get(self, key: NodeKey) -> Optional[Node]:
+        ...
+
+    @overload
+    def get(self, key: NodeKey, default: VT) -> Union[Node, VT]:
+        ...
+
+    def get(
+        self,
+        key: NodeKey,
+        default: Any = None,
+    ) -> Any:
+        tag, name, passedTag = tupleFromNodeKey(key)
+        for node in self.nodes:
+            if name is not None and node.name != name:
+                continue
+            if passedTag and node.tag != tag:
+                continue
+            return node
+        return default
+
+    def __getitem__(self, key: NodeKey) -> Node:
+        tag, name, passedTag = tupleFromNodeKey(key)
+        for node in self.nodes:
+            if name is not None and node.name != name:
+                continue
+            if passedTag and node.tag != tag:
+                continue
+            return node
+        if not passedTag:
+            s = f"name '{name}'"
+        elif name is None:
+            s = f"tag '{tag}'"
+        else:
+            s = f"name '{name}' and tag '{tag}'"
+        raise KeyError(f"No node with {s}.")
+
+    def getAll(
+        self,
+        key: NodeKey,
+    ) -> Iterable[Node]:
+        tag, name, passedTag = tupleFromNodeKey(key)
+        for node in self.nodes:
+            if name is not None and node.name != name:
+                continue
+            if passedTag and node.tag != tag:
+                continue
+            yield node
+
     def __str__(self) -> str:
         return self.print()
 
@@ -289,6 +403,12 @@ def printTag(tag: Optional[str]) -> str:
         return ""
 
 
+def tupleFromNodeKey(key: NodeKey) -> Tuple[Optional[str], Optional[str], bool]:
+    if isinstance(key, str):
+        return None, key, False
+    return key[0], key[1], True
+
+
 def escapedFromRaw(chars: str) -> str:
     return (
         chars.replace("\\", "\\\\")
@@ -303,16 +423,14 @@ def escapedFromRaw(chars: str) -> str:
 
 
 def printValue(val: Any, config: printing.PrintConfig) -> str:
+    if val is None:
+        return "null"
+    if isinstance(val, bool):
+        return "true" if val else "false"
     if isinstance(val, (int, float)):
         return str(val)
     if isinstance(val, str):
         return f'"{escapedFromRaw(val)}"'
-    if val is True:
-        return "true"
-    if val is False:
-        return "false"
-    if val is None:
-        return "null"
     return val.print(config)
 
 
